#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
from peewee import ProgrammingError, DoesNotExist, IntegrityError
import multiprocessing.dummy
from estlex_law.pdfmodel import Kohtulahend, Parsimatu, Legislation, LegislationContent, mydb
from estlex_law.pdfmodel import LegislationContent_CaseContent as LC_CC
from lxml import etree
import text_receiver
import re
import string
from pdfminer.pdfparser import PDFSyntaxError
from pdfminer.pdfdocument import PDFEncryptionError
    
def get_case_nr_from_text(tekst):
    reg_date = re.compile(r'\d{1}\s?[\s-]\s?\d{2}\s?[-\s]\s?\d{1,5}\s?([\(/][\d{11}\d{1}][\(/])?')
    match = reg_date.search(tekst)
    if (match):
        return reformat_casenr(match.group(0))
    else:
        return None
        
def reformat_casenr(str):
    str = str.replace('-', '')
    str = str.replace(' ', '')
    str = str[0] + '-' + str[1:3] + '-' + str[3:len(str)]
    return str
    
def get_date_from_text(tekst):
    reg_date = re.compile(r'\d{1,2}. ?(jaanuar(il)?|veebruar(il)?|mrts(il)?|aprill(il)?|mai(l)?|juuni(l)?|juuli(l)?|august(il)?|septemb(e)?r(il)?|oktoob(e)?r(il)?|novemb(e)?r(il)?|detsemb(e)?r(il)?) \d{4}(.a)?')
    reg_numbers = re.compile(r'\d{1,2}.\d{1,2}.\d{4}.a.')
    match = reg_date.search(tekst)
    match_num = reg_numbers.search(tekst)
    if (match):
        return match.group(0)
    elif(match_num):
        return match_num.group(0)
    else:
        return None

def parse_pub(words, index, keyword, end):
    reg_published = re.compile(r'^Mruse (avalikult teatavaks)?tegemise aeg (ja koht)?$')
    if(reg_published.match(" ".join(words[index:index+6]))):
        return " ".join(words[(index+len(keyword)):(index+len(keyword))+end]).decode('iso-8859-4')
    elif(reg_published.match(" ".join(words[index:index+3]))):
        return " ".join(words[(index+len(keyword))-3:(index+len(keyword))+end]).decode('iso-8859-4')
    elif(reg_published.match(" ".join(words[index:index+5]))):
        return " ".join(words[(index+len(keyword))-1:(index+len(keyword))+end]).decode('iso-8859-4')
            
def pubreg(list):
    reg_published = re.compile(r'Mruse (avalikult teatavaks)?tegemise aeg (ja koht)?')
    return (reg_published.match(" ".join(list[0:6])) or reg_published.match(" ".join(list[0:5])))
            
def is_a_match(keyList, fullList, startIndex, isEnd):
    if(fullList[startIndex] == 'Mruse'):
        return pubreg(fullList[startIndex:len(fullList)])
    keyString = ''.join(keyList)
    keyReg = re.compile(keyString + '(.|,)?')
    matchString = ''.join(fullList[startIndex:startIndex+len(keyList)]).decode('iso-8859-4')
    if(matchString == u'Kohus' and not fullList[startIndex+1][0].isupper()):
        return False
    elif(isEnd == True and (matchString == 'Vrteoasi'.decode('iso-8859-4') or matchString == u'kohtunik' or matchString == u'kohtunik:')):
        return False #TODO: Should actually be left out from text.
    else:
        if (keyReg.match(matchString)):
            return True
        else:
            return False
            
def index_of_end(list, list2, begin):
    for b in range(0, len(list2)):
        ub = list2[b].decode('iso-8859-4')    
        for a in list:
            if a[0] == ub:
                if is_a_match(a, list2, b, True):
                    return begin+b
                elif (b+len(a[0]) < len(list2)):
                    return index_of_end(list, list2[b+1:len(list2)], begin+b+1)
    return len(list2)

def pick_parser(filename):
    tekst = text_receiver.pdfparser(filename)
    
    kohtuotsus_regex = re.compile('(kohtu|resolutiiv)?otsu(s|e)')
    kohtumaarus_regex = re.compile('(kohtu|menetluselpetamise)?mrus'.decode('iso-8859-4'))
    type = "".join(tekst.split()[0:35]).decode('iso-8859-4').lower()
    if(kohtuotsus_regex.search(type)):
        element = Kohtulahend(file_id=filename, full_text=tekst, type=u'Kohtuotsus')
        return parse_kohtulahend(element, tekst, 'otsus')
    elif(kohtumaarus_regex.search(type)):
        element = Kohtulahend(file_id=filename, full_text=tekst, type='Kohtumrus'.decode('iso-8859-4'))
        return parse_kohtulahend(element, tekst, 'maarus')
    elif(re.search('teatis', type)):
        element = Kohtulahend(file_id=filename, full_text=tekst, type=u'Teatis')
        return parse_kohtulahend(element, tekst, 'teatis')
    elif(re.search('kriminaalas', type) or re.search('vrteoas'.decode('iso-8859-4'), type)):
        element = Kohtulahend(file_id=filename, full_text=tekst, type=u'Kohtuotsus')
        return parse_kohtulahend(element, tekst, 'otsus')
    else:
        return Parsimatu(file_id=filename, full_text=tekst)
        
def do_the_actual_parsing(partsdict, tekst):
    parts = partsdict.keys()
    i = 0
    for part in parts:
        parts[i] = part.split()
        i+=1
    words = tekst.split()
    for j in range(0, len(words)):
        uanyword = words[j].decode('iso-8859-4')
        for keyword in parts:
            try:
                if keyword[0] == uanyword:
                    if is_a_match(keyword, words, j, False):
                        end = index_of_end(parts, words[j+len(keyword):len(words)], 0)
                        if(partsdict[" ".join(keyword)] != "" and partsdict[" ".join(keyword)] != None):
                            continue
                        elif(uanyword == 'Mruse'.decode('iso-8859-4')):
                            partsdict[" ".join(keyword)] = parse_pub(words, j, keyword, end)
                        elif(uanyword == 'Sdistatav'.decode('iso-8859-4')):
                            if(words[j+2] == words[j+2].upper()):
                                partsdict[" ".join(keyword)] = " ".join(words[(j+len(keyword)):(j+len(keyword))+end]).decode('iso-8859-4')
                        elif(uanyword == u'kohtunik' or uanyword == u'kohtunik:' or uanyword == 'titmiskohtunik'.decode('iso-8859-4') or uanyword == 'Titmiskohtunik'.decode('iso-8859-4')):
                            if(partsdict['Kohtunik'] != '' or partsdict['kohtunik'] != '' or partsdict['kohtunik:'] != '' or partsdict['titmiskohtunik'.decode('iso-8859-4')] != '' or partsdict['Titmiskohtunik'.decode('iso-8859-4')] != ''):
                                continue
                            else:
                                if((words[j+1] == words[j+1].upper() and words[j+2] == words[j+2].upper()) or (words[j+1] == words[j+1].capitalize() and words[j+2] == words[j+2].capitalize())):
                                    partsdict[" ".join(keyword)] = " ".join(words[(j+len(keyword)):(j+len(keyword))+2]).decode('iso-8859-4')
                                elif(re.match(r'^[a-zA-Z]\.([a-zA-Z])+$', words[j+1])):
                                    partsdict[" ".join(keyword)] = " ".join(words[(j+len(keyword)):(j+len(keyword))+1]).decode('iso-8859-4')
                        elif(uanyword == u'Kohtunik'):
                            if(partsdict['Kohtunik'] != '' or partsdict['kohtunik'] != '' or partsdict['kohtunik:'] != '' or partsdict['titmiskohtunik'.decode('iso-8859-4')] != '' or partsdict['Titmiskohtunik'.decode('iso-8859-4')] != ''):
                                continue
                            else:
                                partsdict[" ".join(keyword)] = " ".join(words[(j+len(keyword)):(j+len(keyword))+end]).decode('iso-8859-4')
                        elif(uanyword == 'Vrteoasi'.decode('iso-8859-4')):
                            partsdict[" ".join(keyword)] = " ".join(words[(j+len(keyword)):(j+len(keyword))+1]).decode('iso-8859-4')
                        else:
                            partsdict[" ".join(keyword)] = " ".join(words[(j+len(keyword)):(j+len(keyword))+end]).decode('iso-8859-4')
            except IndexError:
                print 'perses'
    return partsdict
        
def parse_kohtulahend(element, tekst, type):
    if(re.search(r'kriminaal(a|h)', element.full_text, re.I)): #'Kriminaalhooldusasi' is grouped together with 'kriminaalasi'.
        element.case_type = u'Kriminaalasi'.encode('iso-8859-4')
    elif(re.search(r'vrteoa', element.full_text, re.I)):
        element.case_type = 'Vrteoasi'.decode('iso-8859-4').encode('iso-8859-4')
    elif(re.search(r'tsiviila', element.full_text, re.I)):
        element.case_type = u'Tsiviilasi'.encode('iso-8859-4')
    elif(re.search(r'haldusa', element.full_text, re.I)):
        element.case_type = u'Haldusasi'.encode('iso-8859-4')
    else:
        continue
            
    partsdict = {u'Kohus': '', u'Kohtunik': '', u'kohtunik': '', u'kohtunik:': '',u'Otsuse tegemise aeg ja koht': '', u'otsuse tegemise aeg ja koht': '', u'Kriminaalasja number': '', u'Kohtukoosseis': '', u'Kohtu koosseis': '', 'Kohtuistungi sekretr'.decode('iso-8859-4'): '', 'Kohtuistungisekretr'.decode('iso-8859-4'): '', u'Kriminaalasi': '', 'Prokurr'.decode('iso-8859-4'): '', 'Sdistatav'.decode('iso-8859-4'): '', u'Kaitsja': '', u'RESOLUTSIOON': '', u'Edasikaebamise kord': '', 'SDISTUSE SISU'.decode('iso-8859-4'): '', 'Sdistuse sisu'.decode('iso-8859-4'): '', 'Vrteoasi'.decode('iso-8859-4'): '', u'Kohtuasi': '', u'Menetlustoiming': '', u'Menetlusosalised ja nendeotsus esindajad': '', 'Edasikaebamise kord ning thtaeg'.decode('iso-8859-4'): '', 'Kohtu seisukoht'.decode('iso-8859-4'): '', u't u v a s t a s :': '', 'Mruse avalikult teatavakstegemise aeg ja koht'.decode('iso-8859-4'): '', 'Mruse tegemise aeg ja koht'.decode('iso-8859-4'): '', 'Mruse tegemise aeg'.decode('iso-8859-4'): '', u'Tsiviilasja number': '', u'Resolutsioon': '', 'Menetluse kik ja mruse phjendused'.decode('iso-8859-4'): '', u'Kaitsja': '', 'titmiskohtunik'.decode('iso-8859-4'): '', 'Titmiskohtunik'.decode('iso-8859-4'): ''}
        
            
    partsdict = do_the_actual_parsing(partsdict, tekst)
    
    #Kohus
    if (partsdict.get(u'Kohus')):
        element.court = partsdict.get(u'Kohus').encode('iso-8859-4')
    else:
        if(re.search(r'Maakoh', tekst)):
            if(re.search(r'Harju Maakoh', tekst)):
                element.court = u'Harju Maakohus'.encode('iso-8859-4')
            elif(re.search(r'Prnu Maakoh', tekst)):
                element.court = 'Prnu Maakohus'.decode('iso-8859-4').encode('iso-8859-4')
            elif(re.search(r'Tartu Maakoh', tekst)):
                element.court = u'Tartu Maakohus'.encode('iso-8859-4')
            elif(re.search(r'Viru Maakoh', tekst)):
                element.court = u'Viru Maakohus'.encode('iso-8859-4')
        elif(re.search(r'Halduskoh', tekst)):
            if(re.search(r'Tallinna Halduskoh', tekst)):
                element.court = u'Tallinna Halduskohus'.encode('iso-8859-4')
            elif(re.search(r'Tartu Halduskoh', tekst)):
                element.court = u'Tartu Halduskohus'.encode('iso-8859-4')
        elif(re.search(r'Ringkonnakoh', tekst)):
            if(re.search(r'Tallinna Ringkonnakoh', tekst)):
                element.court = u'Tallinna Ringkonnakoh'.encode('iso-8859-4')
            elif(re.search(r'Tartu Ringkonnakoh', tekst)):
                element.court = u'Tartu Ringkonnakoh'.encode('iso-8859-4')
        elif(re.search(r'Riigikoh', tekst)):
            element.court = u'Riigikohus'.encode('iso-8859-4')
    #Kohtunik
    if (partsdict.get(u'Kohtunik')):
        element.judge = partsdict.get(u'Kohtunik').encode('iso-8859-4')
    elif (partsdict.get(u'kohtunik')):
        element.judge = partsdict.get(u'kohtunik').encode('iso-8859-4')
    elif (partsdict.get(u'kohtunik:')):
        element.judge = partsdict.get(u'kohtunik:').encode('iso-8859-4')
    elif (partsdict.get('titmiskohtunik'.decode('iso-8859-4'))):
        element.judge = partsdict.get('titmiskohtunik'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get('Titmiskohtunik'.decode('iso-8859-4'))):
        element.judge = partsdict.get('Titmiskohtunik'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get(u'Kohtukoosseis')):
        element.judge = partsdict.get(u'Kohtukoosseis').encode('iso-8859-4')
    elif (partsdict.get(u'Kohtu koosseis')):
        element.judge = partsdict.get(u'Kohtu koosseis').encode('iso-8859-4')
    if (element.judge != None and element.judge != ''):
        element.judge = reformat_judge(element.judge)
    #Sekretr
    if (partsdict.get('Kohtuistungi sekretr'.decode('iso-8859-4'))):
        element.secretary = partsdict.get('Kohtuistungi sekretr'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get('Kohtuistungisekretr'.decode('iso-8859-4'))):
        element.secretary = partsdict.get('Kohtuistungisekretr'.decode('iso-8859-4')).encode('iso-8859-4')
    #Prokurr
    if (partsdict.get('Prokurr'.decode('iso-8859-4'))):
        element.prosecutor = partsdict.get('Prokurr'.decode('iso-8859-4')).encode('iso-8859-4')
    #Sdistatav
    if (partsdict.get('Sdistatav'.decode('iso-8859-4'))):    
        element.accused = partsdict.get('Sdistatav'.decode('iso-8859-4')).encode('iso-8859-4')
    #Kaitsja
    if (partsdict.get(u'Kaitsja')):
        element.defence = partsdict.get(u'Kaitsja').encode('iso-8859-4')
    #Vlja antud
    if (partsdict.get('Otsuse tegemise aeg ja koht'.decode('iso-8859-4'))):
        element.published = partsdict.get('Otsuse tegemise aeg ja koht'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get('otsuse tegemise aeg ja koht'.decode('iso-8859-4'))):
        element.published = partsdict.get('otsuse tegemise aeg ja koht'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get('Mruse avalikult teatavakstegemise aeg ja koht'.decode('iso-8859-4'))):
        element.published = partsdict.get('Mruse avalikult teatavakstegemise aeg ja koht'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get('Mruse tegemise aeg ja koht'.decode('iso-8859-4'))):
        element.published = partsdict.get('Mruse tegemise aeg ja koht'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get('Mruse tegemise aeg'.decode('iso-8859-4'))):
        element.published = partsdict.get('Mruse tegemise aeg'.decode('iso-8859-4')).encode('iso-8859-4')
    else:
        element.published = get_date_from_text(tekst)
    #Number
    element.case_nr = get_case_nr_from_text(tekst)
    #Asja kirjeldus
    if (partsdict.get(u'Kriminaalasi')):
        element.case_desc = partsdict.get(u'Kriminaalasi').encode('iso-8859-4')
    elif (partsdict.get(u'Kohtuasi')):
        element.case_desc = partsdict.get(u'Kohtuasi').encode('iso-8859-4')
    #Resolutsioon
    if (partsdict.get(u'RESOLUTSIOON')):
        element.resolution = partsdict.get(u'RESOLUTSIOON').encode('iso-8859-4')
    elif (partsdict.get(u'Resolutsioon')):
        element.resolution = partsdict.get(u'Resolutsioon').encode('iso-8859-4')
    #Apellatsioon
    if (partsdict.get(u'Edasikaebamise kord')):
        element.apellation = partsdict.get(u'Edasikaebamise kord').encode('iso-8859-4')
    elif (partsdict.get('Edasikaebamise kord ning thtaeg'.decode('iso-8859-4'))):
        element.apellation = partsdict.get('Edasikaebamise kord ning thtaeg'.decode('iso-8859-4')).encode('iso-8859-4')
    #Sisu
    if (partsdict.get('SDISTUSE SISU'.decode('iso-8859-4'))):
        element.content = partsdict.get('SDISTUSE SISU'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get('Sdistuse sisu'.decode('iso-8859-4'))):
        element.content = partsdict.get('Sdistuse sisu'.decode('iso-8859-4')).encode('iso-8859-4')
    elif (partsdict.get(u't u v a s t a s :')):
        element.content = partsdict.get(u't u v a s t a s :').encode('iso-8859-4')
    elif (partsdict.get('Menetluse kik ja mruse phjendused'.decode('iso-8859-4'))):
        element.content = partsdict.get('Menetluse kik ja mruse phjendused'.decode('iso-8859-4')).encode('iso-8859-4')
    #Osalised
    if(partsdict.get(u'Menetlusosalised ja nende esindajad')):
        element.participants = partsdict.get(u'Menetlusosalised ja nende esindajad').encode('iso-8859-4')
    return element
    
def reformat_judge(str):
    parts = [x.capitalize() for x in str.split()]
    if(parts[0] == 'Kohtunik'):
        parts = parts[1:len(parts)]
    elif(parts[0] == 'Sekretr'):
        parts = parts[1:len(parts)-2]
    if(len(parts) > 2):
        for i in range(2,len(parts)):
            if(parts[i].startswith('Konsultant') or parts[i].startswith('Sekretr') or parts[i].startswith('Asja') or parts[i].startswith('Vrteo') or parts[i].startswith('Tlk') or parts[i].startswith('Kohtuistung') or parts[i].startswith('Avalduse') or parts[i].startswith('Kriminaal') or parts[i].startswith('Taotluse') or parts[i].startswith('Apelleeritud') or parts[i].startswith('Kirjalikus') or parts[i].startswith('Kaebaja') or parts[i].startswith('(kirjalikus') or parts[i].startswith('Menetlusalune') or parts[i].startswith('Vaidlustatud')):
                parts = parts[:i]
                break
    if (parts[-1][-1] == ','):
        parts[-1] = parts[-1][:-1]
    for part in parts:
        if '-' in part:
            part = '-'.join([y.capitalize() for y in part.split('-')])
        if '.' in part:
            part = ' '.join([z.capitalize() for z in part.split('.')])
    return ' '.join(parts)         
    
def start():
    Kohtulahend.drop_table(fail_silently=True)
    Kohtulahend.create_table()
    Parsimatu.drop_table(fail_silently=True)
    Parsimatu.create_table()
    
    listOfFiles = os.listdir(os.path.join(os.path.realpath(__file__).rsplit('/', 3)[0], 'pdf/full'))
    for nr in listOfFiles:
        
        filename = os.path.join(os.path.realpath(__file__).rsplit('/', 3)[0], 'pdf/full/') + nr
        try:
            print filename
            naide = pick_parser(filename)
            naide.save()
        except PDFSyntaxError:
            naide = Parsimatu(file_id=filename, full_text='Unable to open file')
            naide.save()
            continue
        except PDFEncryptionError:
            naide = Parsimatu(file_id=filename, full_text='File password protected')
            naide.save()
            continue
        except IntegrityError:
            naide.delete_instance()
        except Exception as e:
            naide = Parsimatu(file_id=filename, full_text=str(e))
            naide.save()
            continue
   
#This part is for cross-referencing laws and cases
#Ristviidete loomine
#Written by Johannes Dahl, modified by Brait ispuu
def build_cases_and_laws_together():
    #This part intentionally left out for business purposes.
    #See osa on meelega vlja jetud risaladuse tttu.
    pass
        
def main():
    if len(sys.argv) == 1:
        arg = 'do'
    else:
        arg = sys.argv[1]

    if arg == 'do':
        start()
    elif arg == 'crossref':
        build_cases_and_laws_together()
    else:
        print 'unrecognized command'
