#!/usr/bin/python
# -*- coding: utf-8 -*-

from peewee import *
#Database variables are defined in the settings file (not included)
from estlex_law.settings import DB
from datetime import datetime

class TimestampField(DateTimeField):
    db_field = 'timestamp'

MySQLDatabase.register_fields({'timestamp': 'TIMESTAMP'})

mydb = MySQLDatabase(DB['db'], user=DB['user'], password=DB['password'])
mydb.connect()

class BaseModel(Model):
    class Meta:
        database = mydb
    modified = TimestampField()

class Kohtulahend(BaseModel):
    type = CharField()
    file_id = CharField(unique=True)
    court = CharField(null=True)
    judge = CharField(null=True)
    secretary = CharField(null=True)
    prosecutor = CharField(null=True)
    accused = CharField(null=True)
    defence = CharField(null=True)
    published = CharField(null=True)
    case_nr = CharField(null=True)
    case_desc = CharField(null=True)
    case_act = CharField(null=True)
    case_type = CharField(null=True)
    participants = TextField(null=True)
    resolution = TextField(null=True)
    apellation = TextField(null=True)
    content = TextField(null=True)
    full_text = TextField(null=True)
    created_date = DateTimeField(default=datetime.now)
    
class Parsimatu(BaseModel):
    file_id = CharField(unique=True)
    full_text = TextField(null=True)
    created_date = DateTimeField(default=datetime.now)
    
class Legislation(BaseModel):
    name = CharField(unique=True)
    origin = CharField(unique=True)
    filename = CharField(unique=True)
    xml_file_size = IntegerField(null=True)
    created_date = DateTimeField(default=datetime.now)
    rt_id = CharField(null=True)
    ma_lyhend = CharField(unique=True, null=True)
    ma_marksona = CharField(null=True)
    ma_valjaandja = CharField(null=True)
    ma_dokument_liik = CharField(null=True)
    ma_tekstiliik = CharField(null=True)
    ma_kehtivuse_algus = DateTimeField(null=True)
    ma_kehtivuse_lopp = DateTimeField(null=True)
    embedded_files = CharField(null=True)

class LegislationContent(BaseModel):
    legislation_rt_id = CharField()
    lyhend = CharField()
    section_no = IntegerField(null=True)
    section_ss = IntegerField(null=True)
    subsec_no = IntegerField(null=True)
    subsec_ss = IntegerField(null=True)
    clause_no = IntegerField(null=True)
    clause_ss = IntegerField(null=True)
    text = TextField(null=True)
    heading = CharField(null=True)

    class Meta:
        indexes = (
            (('lyhend', 'section_no', 'subsec_no', 'clause_no'), False),
        )

class LegislationContent_CaseContent(BaseModel):
    legislationcontent_id = IntegerField()
    casecontent_id = IntegerField()
    original_phrase = CharField()
    source = CharField()
    in_resolution = BooleanField()

    class Meta:
        indexes = (
            (('legislationcontent_id', 'casecontent_id'), True),
        )

mydb.create_tables([Kohtulahend, Parsimatu, Legislation], safe=True)